from base64 import urlsafe_b64decode, urlsafe_b64encode
from django.dispatch import receiver
from django.db import models
from ecdsa import NIST256p, SigningKey
from django.db.models.signals import post_save, post_delete
from djangoldp_notification.models import Notification


@receiver(post_save, sender=Notification)
def send_webpush_on_notification(sender, instance, created, **kwargs):
    from .views import _send_push
    if instance.summary :
        if instance.user:   # if the person exists and accepts notifications

            try:
                # local author
                if instance.author.startswith(settings.SITE_URL):
                    who = str(Model.resolve_id(instance.author.replace(settings.SITE_URL, '')).get_full_name())
                # external author
                else:
                    who = requests.get(instance.author).json()['name']
            except:
                who = "Quelqu'un"
                
            payload = {
                "head": "Notification from " + who,
                "body": instance.summary,
                "id": instance.user.id
            }   # make the payload that we wanna send?

            _send_push(payload)


class VAPIDKeyset(models.Model):
    _private_key = models.BinaryField(max_length=43)
    
    @property
    def private_key(self):
        if isinstance(self._private_key, memoryview):
            return self._private_key.tobytes()
        return self._private_key

    def __str__(self):
        return "public_key:{}... private_key:{}...".format(
            self.public_key[:10], self.private_key[:10]
        )

    @property
    def public_key(self):
        key_str = self.private_key
        padding = len(key_str) % 4
        key_str += b"=" * padding
        key = SigningKey.from_string(
            urlsafe_b64decode(key_str), curve=NIST256p
        ).get_verifying_key()
        return urlsafe_b64encode(b"\x04" + key.to_string()).strip(b"=")
